package net.ihe.gazelle.lib.hl7templatespackager.peripherals;

import net.ihe.gazelle.lib.hl7templatespackager.adapters.ProcessExecutor;
import net.ihe.gazelle.lib.hl7templatespackager.application.*;
import net.ihe.gazelle.lib.hl7templatespackager.application.exception.EnvironementException;
import net.ihe.gazelle.lib.hl7templatespackager.application.exception.PackagerProcessException;
import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.kohsuke.args4j.Option;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.SimpleDateFormat;
import java.util.Date;

public class HL7Templates2GOC {

    private static Logger log = null;

    static final String DOCUMENTATION_STRING = "\nHL7Templates2GOC -out outputFolder -bbr buildBlockRepositoryURL";

    private static final SimpleDateFormat formatter = new SimpleDateFormat("dd-MM-yyyy-HH-mm-ss");

    @Option(name = "-out", metaVar = "outputFile", usage = "Output folder", required = true)
    private String outputFile;

    @Option(name = "-in", metaVar = "inputFile", usage = "Input folder", required = false)
    private String inputFile;

    @Option(name = "-bbr", metaVar = "bbr", usage = "Output folder", required = true)
    private String bbr;

    @Option(name = "-serviceName", metaVar = "serviceName", usage = "Validation Service Name", required = false)
    private String serviceName;

    @Option(name = "-prop", metaVar = "properties", usage = "Path to customized java imports file", required = false)
    private String properties;

    @Option(name = "-mvn", metaVar = "mvn", usage = "maven executable path", required = false)
    private String mvnExecPath = "mvn";

    @Option(name = "-notoverride", usage = "Not override the modelDefinition.xml file", required = false)
    @Deprecated
    private Boolean notOverride = false;

    @Option(name = "-ignoreTemplateIdRequirements", usage = "ignore TemplateId Requirements when generating the constraints (useful for C-CDA " +
            "validation tools)", required = false)
    private Boolean ignoreTemplateIdRequirements = false;

    @Option(name = "-ignoreCdaBasicRequirements", usage = "ignore CDA Basic Requirements when generating the constraints (useful for C-CDA " +
            "validation tools)", required = false)
    private Boolean ignoreCdaBasicRequirements = false;

    @Option(name = "-versionLabel", metaVar = "versionLabel", usage = "versionLabel that will be used during the generation", required = false)
    private String versionLabel;

    @Option(name = "-rootClassName", metaVar = "rootClassName", usage = "Root Class Name", required = false)
    private String rootClassName;

    @Option(name = "-HL7TEMP_CDACONFFOLDERNAME", metaVar = "hl7temp_cdaconffoldername", usage = "hl7temp cda conf folder name", required = false)
    private String HL7TEMP_CDACONFFOLDERNAME = "cdabasic";

    @Option(name="-HL7TEMP_RESOURCES_PATH", metaVar = "HL7TEMP_RESOURCES_PATH", usage = "path to hl7templates resources folder", required = false)
    private String HL7TEMP_RESOURCES_PATH;


    private String generationDate;

    /**
     * @return the rootClassName
     */
    public String getRootClassName() {
        return rootClassName;
    }

    /**
     * @param rootClassName the rootClassName to set
     */
    public void setRootClassName(String rootClassName) {
        this.rootClassName = rootClassName;
    }

    public String getVersionLabel() {
        return versionLabel;
    }

    public void setVersionLabel(String versionLabel) {
        this.versionLabel = versionLabel;
    }

    public Boolean getIgnoreTemplateIdRequirements() {
        return ignoreTemplateIdRequirements;
    }

    public void setIgnoreTemplateIdRequirements(Boolean ignoreTemplateIdRequirements) {
        this.ignoreTemplateIdRequirements = ignoreTemplateIdRequirements;
    }

    /**
     * Getter for the ignoreCdaBasicRequirements property.
     *
     * @return the value of the property.
     */
    public Boolean getIgnoreCdaBasicRequirements() {
        return ignoreCdaBasicRequirements;
    }

    /**
     * Setter for the ignoreCdaBasicRequirements property.
     *
     * @param ignoreCdaBasicRequirements value to set to the property.
     */
    public void setIgnoreCdaBasicRequirements(Boolean ignoreCdaBasicRequirements) {
        this.ignoreCdaBasicRequirements = ignoreCdaBasicRequirements;
    }

    public String getServiceName() {
        return serviceName;
    }

    public void setServiceName(String serviceName) {
        this.serviceName = serviceName;
    }

    public String getMvnExecPath() {
        return mvnExecPath;
    }

    public void setMvnExecPath(String mvnExecPath) {
        this.mvnExecPath = mvnExecPath;
    }

    public String getInputFile() {
        if (inputFile == null) {
            inputFile = getOutputFile() + "/archetypes/";
        }
        return inputFile;
    }

    public void setInputFile(String inputFile) {
        this.inputFile = inputFile;
    }

    public String getProperties() {
        if (properties == null) {
            properties = getOutputFile() + "/archetypes/conf.properties";
        }
        return properties;
    }

    public void setProperties(String properties) {
        this.properties = properties;
    }

    @Deprecated
    public Boolean getNotOverride() {
        return notOverride;
    }

    @Deprecated
    public void setNotOverride(Boolean notOverride) {
        this.notOverride = notOverride;
    }

    public String getOutputFile() {
        return outputFile;
    }

    public void setOutputFile(String outputFile) {
        this.outputFile = outputFile;
    }

    public String getBbr() {
        return bbr;
    }

    public void setBbr(String bbr) {
        this.bbr = bbr;
    }

    public String getHL7TEMP_CDACONFFOLDERNAME() {
        return HL7TEMP_CDACONFFOLDERNAME;
    }

    public void setHL7TEMP_CDACONFFOLDERNAME(String HL7TEMP_CDACONFFOLDERNAME) {
        this.HL7TEMP_CDACONFFOLDERNAME = HL7TEMP_CDACONFFOLDERNAME;
    }

    public String getHL7TEMP_RESOURCES_PATH() {
        return HL7TEMP_RESOURCES_PATH;
    }

    public void setHL7TEMP_RESOURCES_PATH(String HL7TEMP_RESOURCES_PATH) {
        this.HL7TEMP_RESOURCES_PATH = HL7TEMP_RESOURCES_PATH;
    }


    public void execute(String[] args) {
        CmdLineParser parser = new CmdLineParser(this);
        try {
            parser.parseArgument(args);

            addSubDirectoryToOutput("/validator");

            // initialize log variable
            initLog();
            GOCLogger gocLogger = new GOCLoggerImpl(getWorkspaceRoot(getOutputFile()), generationDate);

            ProcessExecutor processExecutor = new ProcessExecutorBash(gocLogger);

            //check environement configuration
            EnvironementChecker environementChecker = new EnvironementCheckerImpl();
            environementChecker.checkMVNPath(mvnExecPath,processExecutor);
            environementChecker.checkJavaVersion();


            HL7TEMP_RESOURCES_PATH = HL7TEMP_RESOURCES_PATH==null?
                    getWorkspaceRoot(outputFile)+"/hl7templates-resources":HL7TEMP_RESOURCES_PATH;

            EnvironementConfiguration.setSystemVariables("HL7TEMP_CDACONFFOLDERNAME",HL7TEMP_CDACONFFOLDERNAME,
             "HL7TEMP_RESOURCES_PATH",HL7TEMP_RESOURCES_PATH,"JAVA_EXEC",System.getProperty("java.home")+"/bin/java");

            EnvironementConfiguration.configure();

            log.info("Generation started at : "+generationDate);

            log.info("Workspace root is : " + getWorkspaceRoot(getOutputFile()));
            log.info("Workspace name is : " + getWorkspaceName(getOutputFile()));

            GOCPackager packager = new GOCPackagerFactory().createGOCPackager(getWorkspaceRoot(getOutputFile()), mvnExecPath, generationDate,
                    gocLogger, processExecutor);

            packager.packageBBRToGOCValidationTool(new Workspace(getWorkspaceName(getOutputFile())),
                    new ValidatorConfiguration(getBbr(), getVersionLabel(), getIgnoreTemplateIdRequirements(), getIgnoreCdaBasicRequirements(),
                            getServiceName()));
        } catch (CmdLineException e) {
            System.out.println("Exception in the execution of command line");
            e.printStackTrace();
        } catch (PackagerProcessException e) {
            log.error("Exception during packaging process", e);
        } catch (EnvironementException e) {
            log.error("A problem in the environement configuration",e);
        }
    }

    /**
     * initialize log4j used variable before start logging<br>
     * And set the generationDate field<br>
     * <b>Variable name:</b> <i>${generationDate}</i><br>
     * <b>Usage:</b> as a system property
     */
    private void initLog(){
        this.generationDate = this.formatter.format(new Date());
        System.setProperty("generationDate",this.generationDate);
        System.setProperty("workspaceRoot",getWorkspaceRoot(getOutputFile()));
        log = LoggerFactory.getLogger(HL7Templates2GOC.class);
    }

    private String getWorkspaceName(String outpuFilePath) {
        return getOutputFile().substring(getOutputFile().lastIndexOf("/") + 1);
    }

    private String getWorkspaceRoot(String outpuFilePath) {
        return getOutputFile().substring(0, getOutputFile().lastIndexOf("/") + 1);
    }

    /**
     * Method to add sub-directories to the provided output path
     * To encapsulate validator and logs in one directory
     * Provide empty string to use the old behavior
     * @param subDirectory sub-directorie(s), started with '/'
     */
    private void addSubDirectoryToOutput(String subDirectory){
        if(subDirectory == null){
            subDirectory = "";
        }
        this.outputFile  = this.outputFile + subDirectory;
    }

    public static void main(String[] args) {
        HL7Templates2GOC hl7Templates2GOC = new HL7Templates2GOC();
        hl7Templates2GOC.execute(args);
    }

}
